#! /bin/bash
#############################################################################
# Oracle.sh, version 1.0, 2011-01-31, F. Kordon
#############################################################################
# This script will launch your model checker with various parameters
# and collect some data to evaluate it.
#
# Please do not touch anyrthing but the parts that are dedicated to you
# in separate scripts. This script will be modified for the final version
# to format results accordingly. THE WAY TOOLS ARE INVOKED WILL NOT CHANGE.
#############################################################################

############################################################################
export BENCHMARK_VERSION="SUMo Model Checking Contest, version 1.0 - 2011-02-25"

export BASE_DIR=`pwd`
export BIN_DIR="$BASE_DIR/bin"
export MODELS_DIR="$BASE_DIR/MODELS"
export RESULT_DIR="$BASE_DIR/RESULTS"
export MODEL_LIST="FMS Kanban MAPK Peterson Philosophers SharedMemory TokenRing"
export CURRENT_OS=`uname -s`

#############################################################################
# Getting parameters from the tool
source "$BASE_DIR/setup.sh" "-init"

#############################################################################
# Checking the tool name
if [ ! "$TOOL_NAME" ]; then
	echo "TOOL_NAME not defined. Please edit file 'setup.sh' and fill it."
	exit 1
fi

#############################################################################
# Parameterizing and cleaning outputs
export OUT_FILE="$TOOL_NAME-output.txt"
export ERR_FILE="$TOOL_NAME-errors.txt"
export LOG_FILE="$TOOL_NAME-log.txt"
export CSV_FILE="$TOOL_NAME-data.csv"

rm -f "$OUT_FILE"
rm -f "$ERR_FILE"
rm -f "$LOG_FILE"
rm -f "$CSV_FILE"

#############################################################################
# Checking parameters from the tool
if [ ! "$TEAM_NAME" ]; then
	echo "TEAM_NAME not defined" >> $LOG_FILE
fi
if [ ! "$INSTITUTION_NAME" ]; then
	echo "INSTITUTION_NAME not defined" >> $LOG_FILE
fi
if [ ! "$COUNTRY_NAME" ]; then
	echo "COUNTRY_NAME not defined" >> $LOG_FILE
fi
if [ ! "$CONTACT_EMAIL" ]; then
	echo "CONTACT_EMAIL not defined" >> $LOG_FILE
fi
if [ ! "$CONTACT_NAME" ]; then
	echo "CONTACT_NAME not defined" >> $LOG_FILE
fi
if [ ! "$CONTACT_PHONE" ]; then
	echo "CONTACT_PHONE not defined" >> $LOG_FILE
fi
if [ ! "$CONTACT_URL" ]; then
	echo "CONTACT_URL not defined" >> $LOG_FILE
fi

#############################################################################
# Checking for confinment
if which time >/dev/null 2>/dev/null ; then
	case "$CURRENT_OS" in
		Linux)
			TIME_CMD="time -f TIME=%e"
			;;
	esac
else
	echo "No command for time measures" >> $LOG_FILE
fi
if which memusage >/dev/null 2>/dev/null ; then
	MEM_CMD="memusage -m"
else
	echo "No command for memory measures" >> $LOG_FILE
fi
#if which ulimit >/dev/null 2>/dev/null ; then
#	LIMIT_CMD="ulimit -m 2097152 -t 3600 -u 64 ;"
#else
#	echo "No command for setting limits" >> $LOG_FILE
#fi

ANALYZE_CMD= # to be defined at analsys time

#############################################################################
# Cleaning result data
rm -rf "$RESULT_DIR"
mkdir "$RESULT_DIR"

#############################################################################
# We check for all models if there is something in the model repository.
# Then, we check if there is a directory for all exepected scalability value.
# we then launch the tool for each model, scalability value and provided
# formulas

# Run in confined and measured mode:
function launch {
	TMP_ERR="tmp-erro-$$"
	case "$1" in
		"-generate")
			$LIMIT_CMD $TIME_CMD $MEM_CMD "$BASE_DIR/setup.sh" "$1" >> "$OUT_FILE" 2>> "$TMP_ERR"
			;;
		"-deadlocks")
			$LIMIT_CMD $TIME_CMD $MEM_CMD "$BASE_DIR/setup.sh" "$1" >> "$OUT_FILE" 2>> "$TMP_ERR"
			;;
		"-check")
			$LIMIT_CMD $TIME_CMD $MEM_CMD "$BASE_DIR/setup.sh" "$1" >> "$OUT_FILE" 2>> "$TMP_ERR"
			;;
		*)
			$LIMIT_CMD $TIME_CMD $MEM_CMD "$BASE_DIR/setup.sh" "$1" >> "$OUT_FILE" 2>> "$TMP_ERR"
			;;
	esac
	if [ "$1" = "-generate" -o "$1" = "-deadlocks"	-o "$1" = "-check" ] ; then
		case "$CURRENT_OS" in
			Darwin)
				TIME="time not available yet"
				;;
			Linux)
				TIME=`grep ^TIME= "$TMP_ERR" | cut -d "=" -f 2`
				;;
		esac
		case "$1" in
			"-generate")
				echo "$MODEL;$SCALE;S;$TIME" >> "$CSV_FILE"
				;;
			"-generate")
				echo "$MODEL;$SCALE;D;$TIME" >> "$CSV_FILE"
				;;
			"-check")
				echo "$MODEL;$SCALE;$FORMULA_FILE;$TIME" >> "$CSV_FILE"
				;;
		esac
	fi
	cat "$TMP_ERR" >> "$ERR_FILE"
}

for MODEL in $MODEL_LIST ; do # We explore all the proposed models
	echo '------------------------------------------------------------' | tee -a "$OUT_FILE"
	export MODEL
	export MODEL_DIR="$MODELS_DIR/$MODEL"
	if [ -d "$MODEL_DIR" ]; then
		export SCALES="`$MODEL_DIR/scale.sh`"
		echo "$TOOL_NAME works on $MODEL" | tee -a "$OUT_FILE"
		echo "	 Starting..." | tee -a "$OUT_FILE"
		launch "-start"
		for SCALE in $SCALES ; do # We exlore all the scaled variations of a model
			echo "	 Running on scale $SCALE" | tee -a "$OUT_FILE"
			echo "	 Generating the state space" | tee -a "$OUT_FILE"
			export SCALE
			launch "-generate" # Generation of the state space
			echo "   Checking for deadlocks" | tee -a "$OUT_FILE"
			launch "-deadlocks" # Generation of the state space
			for FORMULA_FILE in "$MODEL_DIR"/*.formula ; do # for all proposed formulas
				export FORMULA_FILE
				echo "			Applying formula $FORMULA_FILE" | tee -a "$OUT_FILE"
				launch "-check" # Evaluation of the formula on the model
			done
		done
		echo "	 ...Stopping" | tee -a "$OUT_FILE"
		launch "-stop"
	else
		echo "$MODEL not found" | tee -a "$OUT_FILE"
	fi
done

